import discord
from cpl_core.database.context import DatabaseContextABC
from cpl_discord.command import DiscordCommandABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.logging.command_logger import CommandLogger
from bot_data.abc.level_repository_abc import LevelRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class LevelGroup(DiscordCommandABC):

    def __init__(
            self,
            logger: CommandLogger,
            message_service: MessageServiceABC,
            bot: DiscordBotServiceABC,
            client_utils: ClientUtilsServiceABC,
            permission_service: PermissionServiceABC,
            translate: TranslatePipe,
            db_context: DatabaseContextABC,
            levels: LevelRepositoryABC,
            servers: ServerRepositoryABC,
    ):
        DiscordCommandABC.__init__(self)

        self._logger = logger
        self._message_service = message_service
        self._bot = bot
        self._client_utils = client_utils
        self._permissions = permission_service
        self._t = translate
        self._db_context = db_context
        self._levels = levels
        self._servers = servers

        self._logger.trace(__name__, f'Loaded command service: {type(self).__name__}')

    @commands.hybrid_group()
    @commands.guild_only()
    async def level(self, ctx: Context):
        pass

    @level.command(alias='levels')
    @commands.guild_only()
    async def list(self, ctx: Context, wait: int = None):
        self._logger.debug(__name__, f'Received command level list {ctx}')
        if not await self._client_utils.check_if_bot_is_ready_yet_and_respond(ctx):
            return

        if not self._permissions.is_member_moderator(ctx.author):
            await self._message_service.send_ctx_msg(ctx, self._t.transform('common.no_permission_message'))
            self._logger.trace(__name__, f'Finished command level list')
            return

        if ctx.guild is None:
            return

        server = self._servers.get_server_by_discord_id(ctx.guild.id)
        levels = self._levels.get_levels_by_server_id(server.server_id)
        if levels.count() < 1:
            await self._message_service.send_ctx_msg(ctx, self._t.transform('modules.level.error.nothing_found'))
            self._logger.trace(__name__, f'Finished command level list')
            return

        level_name = ''
        xp = ''
        permissions = ''
        for level in levels:
            level_name += f'\n{level.name}'
            xp += f'\n{level.min_xp}'
            permissions += f'\n{level.permissions}'

        embed = discord.Embed(
            title=self._t.transform('modules.level.list.title'),
            description=self._t.transform('modules.level.list.description'),
            color=int('ef9d0d', 16)
        )
        embed.add_field(name=self._t.transform('modules.level.list.name'), value=level_name, inline=True)
        embed.add_field(name=self._t.transform('modules.level.list.min_xp'), value=xp, inline=True)
        embed.add_field(name=self._t.transform('modules.level.list.permission_int'), value=permissions, inline=True)
        await self._message_service.send_ctx_msg(ctx, embed, wait_before_delete=wait)
        self._logger.trace(__name__, f'Finished command level list')
